;############################################################################
;#                Tastaturabfrage und PS/2-Konvertierung                    #
;############################################################################
ErrStatus:
	STY	ysave
	STX	xsave
	.IF LCD320
	LDA #15
	STA ColorP
	LDA LabErrFlags
	BNE ErrStatus0
	LDA #'-'-'0'
ErrStatus0:
	CLC
	ADC #'0'
	LDY #0
	LDX #29
	JSR WriteCharXY
	
	.ELSE
	LDA #4
	STA ColorP
	LDA LabErrFlags
	BNE ErrStatus0
	LDA #'-'-'0'
ErrStatus0:
	CLC
	ADC #'0'
	LDY #0
	LDX #$3B
	JSR WriteCharXY
	.ENDIF	
	JMP ColorRestore

ComStatus:
	STY	ysave
	STX	xsave
	.IF LCD320
	LDA #15
	STA ColorP
	LDA COMCH
	CLC
	ADC #'0'
	LDY #0
	LDX #28
	JSR WriteCharXY
	.ELSE
	LDA #5
	STA ColorP
	LDA COMCH
	CLC
	ADC #'0'
	LDY #0
	LDX #$45
	JSR WriteCharXY
	.ENDIF
	JMP ColorRestore
	
BankStatus:
	STY	ysave
	STX	xsave
	.IF LCD320
	LDA #15
	STA ColorP
	LDA CurrBank
	CLC
	ADC #'0'
	LDY #0
	LDX #19
	JSR WriteCharXY
	.ELSE
	LDA #6
	STA ColorP
	LDA CurrBank
	CLC
	ADC #'0'
	LDY #0
	LDX #$31
	JSR WriteCharXY
	.ENDIF
ColorRestore:
	LDX	xsave
	LDY	ysave
	LDA Color
	STA ColorP
	LDA #0
	RTS
	
CapsStr: .db "CAPS",0
InsStr: .DB "INS",0


; ShiftLock CAPS in Status anzeigen ab $24 bei LCD, $4C bei VGA
CapsStatus:
	LDY #0
	.IF LCD320
	LDA #15
	STA ColorP
	LDX #$24
	.ELSE
	LDA #10
	STA ColorP
	LDX #$4C
	.ENDIF
	STX ScreenIdx
	LDX #0
	STX TextIdx
CapsStatusLoop:
	LDX TextIdx
	LDA CapsStr,X
	BEQ CapsStatusExit		
	LDA CapsFlg
	BNE CapsStatusOn
	LDA #green
	STA ColorP
	LDA #' '
	BNE CapsStatusOff	; immer	
CapsStatusOn:
	LDA CapsStr,X
CapsStatusOff:
	LDX ScreenIdx
	JSR writeCharXY
	INC TextIdx
	INC ScreenIdx
	JMP CapsStatusLoop
CapsStatusExit:
	JMP ColorRestore

; Insert INS in Status anzeigen ab $20 bei LCD, $48 bei VGA

InsertStatus:
	LDY #0
	.IF LCD320
	LDA #15
	STA ColorP
	LDX #$20
	.ELSE
	LDA #13
	STA ColorP
	LDX #$48
	.ENDIF
	STX ScreenIdx
	LDX #0
	STX TextIdx
InsertStatusLoop:
	LDX TextIdx
	LDA InsStr,X
	BEQ InsertStatusExit		
	LDA InsFlg
	BNE InsertStatusOn	
	LDA #green
	STA ColorP
	LDA #' '
	BNE InsertStatusOff	; immer	
InsertStatusOn:
	LDA InsStr,X
InsertStatusOff:
	LDX ScreenIdx
	JSR writeCharXY
	INC TextIdx
	INC ScreenIdx
	JMP InsertStatusLoop
InsertStatusExit:
	JMP ColorRestore

; INSERT-Status auf PS/2-Code hin ndern
InsertToggle:		
	LDA InsFlg
	EOR #$20
	STA InsFlg
	JSR InsertStatus
	LDA #0
	BEQ ReleaseLoop	; immer!

; CAPS-Status auf PS/2-Code hin ndern
CapsToggle:		
	LDA CapsFlg
	EOR #$20
	STA CapsFlg
	JSR CapsStatus
	LDA #0
ReleaseLoop:
	LDY PS2code	; losgelassen?
	BNE ReleaseLoop
	RTS
	
	
SetCOM0:
	LDA 	#0
	BEQ 	SetCOMCH
SetCOM1:
	LDA 	#0
	STA	UART_BAUD
	STA	UART_SEL	; Multiplexer fr ser. Schnittstelle
	LDA 	#1
	BNE 	SetCOMCH
SetCOM2:
	LDA 	#0
	STA	UART_BAUD
	LDA 	#1
	STA	UART_SEL	; Multiplexer fr ser. Schnittstelle
	LDA 	#2
SetCOMCH:
	STA 	COMCH
	JSR 	ComStatus
	LDA 	#0
	RTS
	
	
; konvertiert Scancode in ASCII, bercksichtigt Shift- und Ctrl-Taste
PS2scan:
	LDA PS2code	; E0-Codes haben hier Bit 7 gesetzt, Rest wie letztes Byte
	CMP #0		; KEINE!, kehrt dann mit 0 zurck
	BEQ PS2scanNone
	STA peekScancode
	CMP #$09	; F10
	BEQ SetCOM0
	CMP #$78	; F11
	BEQ SetCOM1
	CMP #$07	; F12
	BEQ SetCOM2

	CMP #$F0	; Insert
	BEQ InsertToggle
	CMP #$F1	; Delete
	BEQ DeleteJmp

	CMP #$58
	BEQ CapsToggleJmp

	LDY Clineh	; Running program? Keine Pfeiltasten u.a.
	INY
	BNE PS2noArrows
	CMP #$F5	; Arrow Up
	BEQ RowUp
	CMP #$F2	; Arrow Down
	BEQ RowDown
	CMP #$EB
	BEQ ColDown	;
	CMP #$F4
	BEQ ColUp
	CMP #$FC
	BEQ ClearScreen
	CMP #$EC
	BEQ HomeCursor
PS2noArrows:
	AND #$7F
	CMP #$0D	; TAB wird COPY, kopiert ASCII-Wert unter Cursor
	BEQ Copy
	TAY
	LDA PS2toASCII,Y	; Default: Unshifted
	BIT kbdFlgs		; Shift-Taste in Bit 7 (=N), Ctrl-Taste in Bit 6 (=V), ESC Bit 5, neg. Logik!
	BVC PS2scanControls
	BMI PS2scanNoShift	
	LDA PS2toASCIIshifted,Y	; Shifted Key
	STA peekkey
	CMP #$1D	; ENTER shifted
	BEQ ShiftReturnJmp
	RTS
PS2scanControls:
	AND #$0F
PS2scanNoShift:
	STA peekkey
	RTS
PS2scanNone:
	LDY #35		; nach losgelassener Taste Timer neu setzen
	STY repeat
	LDY #5
	STY repRate
	RTS

HomeCursor:	
	JSR hc
	JMP CursKeyRel	; restauriert X und Y aus xkey, ykey

ClearScreen:	
	JSR ClearFullScreen
	JMP CursKeyRel
RowUp:	
	JSR vt
	JMP CursKeyRel
RowDown:	
	JSR lf
	JMP CursKeyRel
ColUp:	
	JSR ht
	JMP CursKeyRel
ColDown:
	JSR bs
	JMP CursKeyRel	
DeleteJmp:
	JMP Delete	
CapsToggleJmp: 
	JMP CapsToggle
ShiftReturnJmp:
	JMP ShiftReturn
	
Copy:		; Holt Taste unter dem Cursor
	LDA TextWR
	LDY #5		; miges Repeat ohne Shift
	BIT kbdFlgs	; Shift-Taste in Bit 7 (=N), Ctrl-Taste in Bit 6 (=V), ESC Bit 5, neg. Logik!
	BMI CopyExit	
	LDY #1		; schnelles Repeat mit Shift und Cursor-Tasten
CopyExit:
	STY repRate
	JSR UpCaseConvert ; Taste gedrckt, ggf in Upcase umwandeln
	LDY ykey
	LDX xkey
	CMP #0		; WICHTIG! sonst klappt der 0-Vergleich in EhBASIC nicht
	STA lastkey
	RTS

ShiftReturn:		; Holt ganze Zeile unter dem Cursor ab linkem Rand
IbuffL = IBuffe-IBuffs-1
	LDA #$80
	STA GetbFlg	; Flag
	LDY Clineh	; immediate Mode?
	INY
	BEQ ShiftReturnStart	; wenn ja
	LDA #8
	JSR WriteAcc
	LDA #0
	RTS
ShiftReturnStart:
	LDY #IbuffL+1		; max. Lnge
	CPY #colstotal
	BCC ShiftRetLoop	; wenn IbuffL >= colstotal 
	LDY #colstotal
ShiftRetLoop:
	DEY
	STY TextX
	LDA TextWR
	CMP #' '		; erstes Nicht-Space-zeichen von hinten suchen
	BEQ ShiftRetSpace
	LDX #0
	STX GetbFlg
ShiftRetSpace:
	BIT GetbFlg
	BPL ShiftRetNoSpace
	LDA #0			; mit Nullen fllen
ShiftRetNoSpace:
	STA Ibuffs,Y
	CPY #0
	BNE ShiftRetLoop
	JSR LAB_CRLF		; print CR/LF
	LDY #5			; miges Repeat trotz Shift
	JSR CursTimerSet	
	LDX #<Ibuffs
	LDY #>Ibuffs
	JMP LAB_1280		; execute buffer
	

CursKeyRel:
	LDY #5		; miges Repeat ohne Shift
	BIT kbdFlgs	; Shift-Taste in Bit 7 (=N), Ctrl-Taste in Bit 6 (=V), ESC Bit 5, neg. Logik!
	BMI CursTimerSet	
	LDY #1		; schnelles Repeat mit Shift und Cursor-Tasten
CursTimerSet:
	STY repRate
	JSR RepeatTimer
	BEQ CursKeyRel3
	LDY PS2code	; Cursor, Tab und sonstige Sondertasten losgelassen?
	BNE CursKeyRel
CursKeyRel3:
	LDA #0
	RTS
	
RepeatTimer:
	JSR VSwait
	LDY repeat
	DEC repeat
	BMI RepeatEnd
	RTS
RepeatEnd:
	LDY repRate
	STY repeat
	RTS
		
UpCaseConvert:		; wandelt Akku mit ASCII in Grobuchstaben fr BASIC
	CMP #0	; Cursor, Tab und sonstige Sondertasten
	BEQ UpCaseConvExit
	;weitere hier
	;CMP #$08
	;BNE UpCaseCaps
	;LDA #$7F
UpCaseCaps:
	CMP #$41	; alle von "A" an
	BMI UpCaseConvExit
	CMP #$7B
	BPL UpCaseConvExit
	CMP #$60
	BPL UpCaseConvEOR
	CMP #$5B
	BPL UpCaseConvExit
UpCaseConvEOR:
	EOR CapsFlg
UpCaseConvExit:
	RTS

;############################################################################
; Scanset 2, deutsche Tastatur von carsten Meyer, cm@ix.de
;
; Designed and Written by Daryl Rictor (c) 2001   65c02@altavista.com
; Offered as freeware.  No warranty is given.  Use at your own risk.
;
; Unshifted table for scancodes to ascii conversion
;		        Scan|Keyboard
;		        Code|Key
;		        ----|----------
PS2toASCII:
	.BYTE $00	; 00 no key pressed
	.byte $89	; 01 F9
	.byte $87	; 02 relocated F7
	.byte $85	; 03 F5
	.byte TK_RUN	; 04 F3
	.byte TK_LIST	; 05 F1
	.byte TK_DIR	; 06 F2
	.byte $8C	; 07 F12
	.byte $00	; 08 
	.byte $8A	; 09 F10
	.byte $88	; 0A F8
	.byte $86	; 0B F6
	.byte TK_PRINT	; 0C F4
	.byte $09	; 0D tab
	.byte '^'	; 0E `~
	.byte $8F	; 0F relocated Print Screen key
	.byte $03	; 10 relocated Pause/Break key
	.byte $A0	; 11 left alt (right alt too)
	.byte $00	; 12 left shift
	.byte $E0	; 13 relocated Alt release code
	.byte $00	; 14 left ctrl (right ctrl too)
	.byte $71	; 15 qQ
	.byte $31	; 16 1!
	.byte $00	; 17 
	.byte $00	; 18 
	.byte $00	; 19 
	.byte 'y'	; 1A zZ
	.byte $73	; 1B sS
	.byte $61	; 1C aA
	.byte $77	; 1D wW
	.byte $32	; 1E 2@
	.byte $A1	; 1F Windows 98 menu key (left side)
	.byte $02	; 20 relocated ctrl-break key
	.byte $63	; 21 cC
	.byte $78	; 22 xX
	.byte $64	; 23 dD
	.byte $65	; 24 eE
	.byte $34	; 25 4$
	.byte $33	; 26 3#
	.byte $A2	; 27 Windows 98 menu key (right side)
	.byte $00	; 28
	.byte $20	; 29 space
	.byte $76	; 2A vV
	.byte $66	; 2B fF
	.byte $74	; 2C tT
	.byte $72	; 2D rR
	.byte $35	; 2E 5%
	.byte $A3	; 2F Windows 98 option key (right click, right side)
	.byte $00	; 30
	.byte $6E	; 31 nN
	.byte $62	; 32 bB
	.byte $68	; 33 hH
	.byte $67	; 34 gG
	.byte 'z'	; 35 yY
	.byte $36	; 36 6^
	.byte $00	; 37
	.byte $00	; 38
	.byte $00	; 39
	.byte $6D	; 3A mM
	.byte $6A	; 3B jJ
	.byte $75	; 3C uU
	.byte $37	; 3D 7&
	.byte $38	; 3E 8*
	.byte $00	; 3F
	.byte $00	; 40
	.byte $2C	; 41 ,<
	.byte $6B	; 42 kK
	.byte $69	; 43 iI
	.byte $6F	; 44 oO
	.byte $30	; 45 0)
	.byte $39	; 46 9(
	.byte $00	; 47
	.byte $00	; 48
	.byte $2E	; 49 .>
	.byte '-'	; 4A /?
	.byte $6C	; 4B lL
	.byte '['	; 4C ;:
	.byte $70	; 4D pP
	.byte '\'	; 4E -_
	.byte $00	; 4F
	.byte $00	; 50
	.byte $00	; 51
	.byte ']'	; 52 '"
	.byte $00	; 53
	.byte '~'	; 54  gibts nicht, statt [{
	.byte $27	; 55 ' war =+
	.byte $00	; 56
	.byte $00	; 57
	.byte $00	; 58 caps
	.byte $00	; 59 r shift
	.byte $0D	; 5A <Enter>
	.byte '+'	; 5B ]}
	.byte $00	; 5C
	.byte '#'	; 5D \|
	.byte $00	; 5E
	.byte $00	; 5F
	.byte $00	; 60
	.byte '<'	; 61
	.byte $00	; 62
	.byte $00	; 63
	.byte $00	; 64
	.byte $00	; 65
	.byte $08	; 66 bkspace
	.byte $00	; 67
	.byte $00	; 68
	.byte $31	; 69 kp 1
	.byte '/'	; 6A kp / converted from E04A in code
	.byte $34	; 6B kp 4
	.byte $37	; 6C kp 7
	.byte $00	; 6D
	.byte $00	; 6E
	.byte $00	; 6F
	.byte $30	; 70 kp 0
	.byte $2E	; 71 kp .
	.byte $32	; 72 kp 2
	.byte $35	; 73 kp 5
	.byte $36	; 74 kp 6
	.byte $38	; 75 kp 8
	.byte $1B	; 76 esc
	.byte $00	; 77 num lock
	.byte $8B	; 78 F11
	.byte $2B	; 79 kp +
	.byte $33	; 7A kp 3
	.byte $2D	; 7B kp -
	.byte $2A	; 7C kp *
	.byte $39	; 7D kp 9
	.byte $8D	; 7E scroll lock
	.byte $00	; 7F 
;
;############################################################################
;
; Table for shifted scancodes 
;
PS2toASCIIshifted:
	.byte $00	; 80 
	.byte $C9	; 81 F9
	.byte $C7	; 82 relocated F7 
	.byte $C5	; 83 F5 (F7 actual scancode=83)
	.byte $C3	; 84 F3
	.byte $C1	; 85 F1
	.byte $C2	; 86 F2
	.byte $CC	; 87 F12
	.byte $00	; 88 
	.byte $CA	; 89 F10
	.byte $C8	; 8A F8
	.byte $C6	; 8B F6
	.byte $C4	; 8C F4
	.byte $09	; 8D tab
	.byte $7E	; 8E `~
	.byte $CF	; 8F relocated Print Screen key
	.byte $03	; 90 relocated Pause/Break key
	.byte $A0	; 91 left alt (right alt)
	.byte $00	; 92 left shift
	.byte $E0	; 93 relocated Alt release code
	.byte $00	; 94 left ctrl (and right ctrl)
	.byte $51	; 95 qQ
	.byte $21	; 96 1!
	.byte $00	; 97 
	.byte $00	; 98 
	.byte $00	; 99 
	.byte 'Y'	; 9A zZ
	.byte $53	; 9B sS
	.byte $41	; 9C aA
	.byte $57	; 9D wW
	.byte '"'	; 9E 2@
	.byte $E1	; 9F Windows 98 menu key (left side)
	.byte $02	; A0 relocated ctrl-break key
	.byte $43	; A1 cC
	.byte $58	; A2 xX
	.byte $44	; A3 dD
	.byte $45	; A4 eE
	.byte $24	; A5 4$
	.byte $40	; A6 3#
	.byte $E2	; A7 Windows 98 menu key (right side)
	.byte $00	; A8
	.byte $20	; A9 space
	.byte $56	; AA vV
	.byte $46	; AB fF
	.byte $54	; AC tT
	.byte $52	; AD rR
	.byte $25	; AE 5%
	.byte $E3	; AF Windows 98 option key (right click, right side)
	.byte $00	; B0
	.byte $4E	; B1 nN
	.byte $42	; B2 bB
	.byte $48	; B3 hH
	.byte $47	; B4 gG
	.byte 'Z'	; B5 yY
	.byte '&'	; B6 6^
	.byte $00	; B7
	.byte $00	; B8
	.byte $00	; B9
	.byte $4D	; BA mM
	.byte $4A	; BB jJ
	.byte $55	; BC uU
	.byte '/'	; BD 7&
	.byte '('	; BE 8*
	.byte $00	; BF
	.byte $00	; C0
	.byte ';'	; C1 ,<
	.byte $4B	; C2 kK
	.byte $49	; C3 iI
	.byte $4F	; C4 oO
	.byte '='	; C5 0)
	.byte ')'	; C6 9(
	.byte $00	; C7
	.byte $00	; C8
	.byte ':'	; C9 .>
	.byte '_'	; CA /?
	.byte $4C	; CB lL
	.byte '{'	; CC ;:
	.byte $50	; CD pP
	.byte '?'	; CE -_
	.byte $00	; CF
	.byte $00	; D0
	.byte $00	; D1
	.byte '}'	; D2 '"
	.byte $00	; D3
	.byte ''	; D4 [{
	.byte '|'	; D5 =+
	.byte $00	; D6
	.byte $00	; D7
	.byte $00	; D8 caps
	.byte $00	; D9 r shift
	.byte $1D	; DA <Enter> shifted!
	.byte '*'	; DB ]}
	.byte $00	; DC
	.byte $27	; DD \|
	.byte $00	; DE
	.byte $00	; DF
	.byte $00	; E0
	.byte '>'	; E1
	.byte $00	; E2
	.byte $00	; E3
	.byte $00	; E4
	.byte $00	; E5
	.byte $08	; E6 bkspace
	.byte $00	; E7
	.byte $00	; E8
	.byte '1'	; E9 kp 1
	.byte '/'	; EA kp / converted from E04A in code
	.byte '4'	; EB kp 4
	.byte '7'	; EC kp 7
	.byte $00	; ED
	.byte $00	; EE
	.byte $00	; EF
	.byte '0'	; F0 kp 0
	.byte ','	; F1 kp .
	.byte '2'	; F2 kp 2
	.byte '5'	; F3 kp 5
	.byte '6'	; F4 kp 6
	.byte '8'	; F5 kp 8
	.byte $1B	; F6 esc
	.byte $00	; F7 num lock
	.byte $CB	; F8 F11
	.byte '+'	; F9 kp +
	.byte '3'	; FA kp 3
	.byte '-'	; FB kp -
	.byte '*'	; FC kp *
	.byte '9'	; FD kp 9
	.byte $CD	; FE scroll lock
	.BYTE $00	; FF 
	
;############################################################################
